@@ -23,11 +23,15 @@
 
 #include <string.h>
 
+#define G_UDEV_API_IS_SUBJECT_TO_CHANGE
+#include <gudev/gudev.h>
+
 #include "NetworkManager.h"
 #include "nm-device-ethernet.h"
 #include "nm-device-wifi.h"
 #include "nm-gsm-device.h"
 #include "nm-cdma-device.h"
+#include "nm-device-bt.h"
 #include "nm-device.h"
 #include "nm-device-private.h"
 #include "nm-object-private.h"
@@ -53,7 +57,13 @@ typedef struct {
 	gboolean null_ip4_config;
 	NMDHCP4Config *dhcp4_config;
 	gboolean null_dhcp4_config;
+	NMIP6Config *ip6_config;
+	gboolean null_ip6_config;
+	NMDHCP6Config *dhcp6_config;
+	gboolean null_dhcp6_config;
 	NMDeviceState state;
+
+	GUdevClient *client;
 	char *product;
 	char *vendor;
 } NMDevicePrivate;
@@ -67,9 +77,11 @@ enum {
 	PROP_MANAGED,
 	PROP_IP4_CONFIG,
 	PROP_DHCP4_CONFIG,
+	PROP_IP6_CONFIG,
 	PROP_STATE,
 	PROP_PRODUCT,
 	PROP_VENDOR,
+	PROP_DHCP6_CONFIG,
 
 	LAST_PROP
 };
@@ -89,7 +101,6 @@ nm_device_init (NMDevice *device)
 	NMDevicePrivate *priv = NM_DEVICE_GET_PRIVATE (device);
 
 	priv->state = NM_DEVICE_STATE_UNKNOWN;
-	priv->disposed = FALSE;
 }
 
 static gboolean
@@ -172,6 +183,86 @@ demarshal_dhcp4_config (NMObject *object, GParamSpec *pspec, GValue *value, gpoi
 	return TRUE;
 }
 
+static gboolean
+demarshal_ip6_config (NMObject *object, GParamSpec *pspec, GValue *value, gpointer field)
+{
+	NMDevicePrivate *priv = NM_DEVICE_GET_PRIVATE (object);
+	const char *path;
+	NMIP6Config *config = NULL;
+	DBusGConnection *connection;
+
+	if (!G_VALUE_HOLDS (value, DBUS_TYPE_G_OBJECT_PATH))
+		return FALSE;
+
+	priv->null_ip6_config = FALSE;
+
+	path = g_value_get_boxed (value);
+	if (path) {
+		if (!strcmp (path, "/"))
+			priv->null_ip6_config = TRUE;
+		else {
+			config = NM_IP6_CONFIG (_nm_object_cache_get (path));
+			if (config)
+				config = g_object_ref (config);
+			else {
+				connection = nm_object_get_connection (object);
+				config = NM_IP6_CONFIG (nm_ip6_config_new (connection, path));
+			}
+		}
+	}
+
+	if (priv->ip6_config) {
+		g_object_unref (priv->ip6_config);
+		priv->ip6_config = NULL;
+	}
+
+	if (config)
+		priv->ip6_config = config;
+
+	_nm_object_queue_notify (object, NM_DEVICE_IP6_CONFIG);
+	return TRUE;
+}
+
+static gboolean
+demarshal_dhcp6_config (NMObject *object, GParamSpec *pspec, GValue *value, gpointer field)
+{
+	NMDevicePrivate *priv = NM_DEVICE_GET_PRIVATE (object);
+	const char *path;
+	NMDHCP6Config *config = NULL;
+	DBusGConnection *connection;
+
+	if (!G_VALUE_HOLDS (value, DBUS_TYPE_G_OBJECT_PATH))
+		return FALSE;
+
+	priv->null_dhcp6_config = FALSE;
+
+	path = g_value_get_boxed (value);
+	if (path) {
+		if (!strcmp (path, "/"))
+			priv->null_dhcp6_config = TRUE;
+		else {
+			config = NM_DHCP6_CONFIG (_nm_object_cache_get (path));
+			if (config)
+				config = g_object_ref (config);
+			else {
+				connection = nm_object_get_connection (object);
+				config = NM_DHCP6_CONFIG (nm_dhcp6_config_new (connection, path));
+			}
+		}
+	}
+
+	if (priv->dhcp6_config) {
+		g_object_unref (priv->dhcp6_config);
+		priv->dhcp6_config = NULL;
+	}
+
+	if (config)
+		priv->dhcp6_config = config;
+
+	_nm_object_queue_notify (object, NM_DEVICE_DHCP6_CONFIG);
+	return TRUE;
+}
+
 static void
 register_for_property_changed (NMDevice *device)
 {
@@ -182,8 +273,10 @@ register_for_property_changed (NMDevice *device)
 		{ NM_DEVICE_DRIVER,       _nm_object_demarshal_generic, &priv->driver },
 		{ NM_DEVICE_CAPABILITIES, _nm_object_demarshal_generic, &priv->capabilities },
 		{ NM_DEVICE_MANAGED,      _nm_object_demarshal_generic, &priv->managed },
-		{ NM_DEVICE_IP4_CONFIG,   demarshal_ip4_config,        &priv->ip4_config },
-		{ NM_DEVICE_DHCP4_CONFIG, demarshal_dhcp4_config,      &priv->dhcp4_config },
+		{ NM_DEVICE_IP4_CONFIG,   demarshal_ip4_config,         &priv->ip4_config },
+		{ NM_DEVICE_DHCP4_CONFIG, demarshal_dhcp4_config,       &priv->dhcp4_config },
+		{ NM_DEVICE_IP6_CONFIG,   demarshal_ip6_config,         &priv->ip6_config },
+		{ NM_DEVICE_DHCP6_CONFIG, demarshal_dhcp6_config,       &priv->dhcp6_config },
 		{ NULL },
 	};
 
@@ -267,6 +360,12 @@ dispose (GObject *object)
 		g_object_unref (priv->ip4_config);
 	if (priv->dhcp4_config)
 		g_object_unref (priv->dhcp4_config);
+	if (priv->ip6_config)
+		g_object_unref (priv->ip6_config);
+	if (priv->dhcp6_config)
+		g_object_unref (priv->dhcp6_config);
+	if (priv->client)
+		g_object_unref (priv->client);
 
 	G_OBJECT_CLASS (nm_device_parent_class)->dispose (object);
 }
@@ -315,6 +414,12 @@ get_property (GObject *object,
 	case PROP_DHCP4_CONFIG:
 		g_value_set_object (value, nm_device_get_dhcp4_config (device));
 		break;
+	case PROP_IP6_CONFIG:
+		g_value_set_object (value, nm_device_get_ip6_config (device));
+		break;
+	case PROP_DHCP6_CONFIG:
+		g_value_set_object (value, nm_device_get_dhcp6_config (device));
+		break;
 	case PROP_STATE:
 		g_value_set_uint (value, nm_device_get_state (device));
 		break;
@@ -361,13 +466,13 @@ nm_device_class_init (NMDeviceClass *device_class)
 	/**
 	 * NMDevice:udi:
 	 *
-	 * The HAL UDI of the device.
+	 * The Unique Device Identifier of the device.
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_UDI,
 		 g_param_spec_string (NM_DEVICE_UDI,
 						  "UDI",
-						  "HAL UDI",
+						  "Unique Device Identifier",
 						  NULL,
 						  G_PARAM_READABLE));
 
@@ -436,6 +541,32 @@ nm_device_class_init (NMDeviceClass *device_class)
 						  NM_TYPE_DHCP4_CONFIG,
 						  G_PARAM_READABLE));
 
+	/**
+	 * NMDevice:ip6-config:
+	 *
+	 * The #NMIP6Config of the device.
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_IP6_CONFIG,
+		 g_param_spec_object (NM_DEVICE_IP6_CONFIG,
+		                      "IP6 Config",
+		                      "IP6 Config",
+		                      NM_TYPE_IP6_CONFIG,
+		                      G_PARAM_READABLE));
+
+	/**
+	 * NMDevice:dhcp6-config:
+	 *
+	 * The #NMDHCP6Config of the device.
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_DHCP6_CONFIG,
+		 g_param_spec_object (NM_DEVICE_DHCP6_CONFIG,
+						  "DHCP6 Config",
+						  "DHCP6 Config",
+						  NM_TYPE_DHCP6_CONFIG,
+						  G_PARAM_READABLE));
+
 	/**
 	 * NMDevice:state:
 	 *
@@ -549,6 +680,9 @@ nm_device_new (DBusGConnection *connection, const char *path)
 	case NM_DEVICE_TYPE_CDMA:
 		dtype = NM_TYPE_CDMA_DEVICE;
 		break;
+	case NM_DEVICE_TYPE_BT:
+		dtype = NM_TYPE_DEVICE_BT;
+		break;
 	default:
 		g_warning ("Unknown device type %d", g_value_get_uint (&value));
 		break;
@@ -596,10 +730,11 @@ nm_device_get_iface (NMDevice *device)
  * nm_device_get_udi:
  * @device: a #NMDevice
  *
- * Gets the HAL UDI of the #NMDevice.
+ * Gets the Unique Device Identifier of the #NMDevice.
  *
- * Returns: the HAL UDI of the device. This is the internal string used by the
- * device, and must not be modified.
+ * Returns: the Unique Device Identifier of the device.  This identifier may be
+ * used to gather more information about the device from various operating
+ * system services like udev or sysfs.
  **/
 const char *
 nm_device_get_udi (NMDevice *device)
@@ -763,6 +898,75 @@ nm_device_get_dhcp4_config (NMDevice *device)
 	return priv->dhcp4_config;
 }
 
+/**
+ * nm_device_get_ip6_config:
+ * @device: a #NMDevice
+ *
+ * Gets the current #NMIP6Config associated with the #NMDevice.
+ *
+ * Returns: the #NMIP6Config or %NULL if the device is not activated.
+ **/
+NMIP6Config *
+nm_device_get_ip6_config (NMDevice *device)
+{
+	NMDevicePrivate *priv;
+	char *path;
+	GValue value = { 0, };
+
+	g_return_val_if_fail (NM_IS_DEVICE (device), NULL);
+
+	priv = NM_DEVICE_GET_PRIVATE (device);
+	if (priv->ip6_config)
+		return priv->ip6_config;
+	if (priv->null_ip6_config)
+		return NULL;
+
+	path = _nm_object_get_object_path_property (NM_OBJECT (device), NM_DBUS_INTERFACE_DEVICE, "Ip6Config");
+	if (path) {
+		g_value_init (&value, DBUS_TYPE_G_OBJECT_PATH);
+		g_value_take_boxed (&value, path);
+		demarshal_ip6_config (NM_OBJECT (device), NULL, &value, &priv->ip6_config);
+		g_value_unset (&value);
+	}
+
+	return priv->ip6_config;
+}
+
+/**
+ * nm_device_get_dhcp6_config:
+ * @device: a #NMDevice
+ *
+ * Gets the current #NMDHCP6Config associated with the #NMDevice.
+ *
+ * Returns: the #NMDHCPConfig or %NULL if the device is not activated or not
+ * using DHCP.
+ **/
+NMDHCP6Config *
+nm_device_get_dhcp6_config (NMDevice *device)
+{
+	NMDevicePrivate *priv;
+	char *path;
+	GValue value = { 0, };
+
+	g_return_val_if_fail (NM_IS_DEVICE (device), NULL);
+
+	priv = NM_DEVICE_GET_PRIVATE (device);
+	if (priv->dhcp6_config)
+		return priv->dhcp6_config;
+	if (priv->null_dhcp6_config)
+		return NULL;
+
+	path = _nm_object_get_object_path_property (NM_OBJECT (device), NM_DBUS_INTERFACE_DEVICE, "Dhcp6Config");
+	if (path) {
+		g_value_init (&value, DBUS_TYPE_G_OBJECT_PATH);
+		g_value_take_boxed (&value, path);
+		demarshal_dhcp6_config (NM_OBJECT (device), NULL, &value, &priv->dhcp6_config);
+		g_value_unset (&value);
+	}
+
+	return priv->dhcp6_config;
+}
+
 /**
  * nm_device_get_state:
  * @device: a #NMDevice
@@ -788,179 +992,127 @@ nm_device_get_state (NMDevice *device)
 	return priv->state;
 }
 
-static char *
-get_ancestor_device (NMDevice *device,
-                     DBusGConnection *connection,
-                     const char *udi,
-                     gboolean want_origdev)
+/* From hostap, Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi> */
+
+static int hex2num (char c)
 {
-	DBusGProxy *proxy;
-	GError *err = NULL;
-	char *parent = NULL;
+	if (c >= '0' && c <= '9')
+		return c - '0';
+	if (c >= 'a' && c <= 'f')
+		return c - 'a' + 10;
+	if (c >= 'A' && c <= 'F')
+		return c - 'A' + 10;
+	return -1;
+}
 
-	g_return_val_if_fail (connection != NULL, NULL);
-	g_return_val_if_fail (udi != NULL, NULL);
+static int hex2byte (const char *hex)
+{
+	int a, b;
+	a = hex2num(*hex++);
+	if (a < 0)
+		return -1;
+	b = hex2num(*hex++);
+	if (b < 0)
+		return -1;
+	return (a << 4) | b;
+}
 
-	proxy = dbus_g_proxy_new_for_name (connection, "org.freedesktop.Hal", udi, "org.freedesktop.Hal.Device");
-	if (!proxy)
+/* End from hostap */
+
+static char *
+get_decoded_property (GUdevDevice *device, const char *property)
+{
+	const char *orig, *p;
+	char *unescaped, *n;
+	guint len;
+
+	p = orig = g_udev_device_get_property (device, property);
+	if (!orig)
 		return NULL;
 
-	if (want_origdev) {
-		gboolean serial = FALSE;
-
-		if (NM_IS_GSM_DEVICE (device) || NM_IS_CDMA_DEVICE (device))
-			serial = TRUE;
-
-		dbus_g_proxy_call (proxy, "GetPropertyString", NULL,
-		                   G_TYPE_STRING, serial ? "serial.originating_device" : "net.originating_device",
-		                   G_TYPE_INVALID,
-		                   G_TYPE_STRING, &parent,
-		                   G_TYPE_INVALID);
-
-		if (!parent) {
-			/* Older HAL uses 'physical_device' */
-			dbus_g_proxy_call (proxy, "GetPropertyString", &err,
-			                   G_TYPE_STRING, serial ? "serial.physical_device" : "net.physical_device",
-			                   G_TYPE_INVALID,
-			                   G_TYPE_STRING, &parent,
-			                   G_TYPE_INVALID);
+	len = strlen (orig);
+	n = unescaped = g_malloc0 (len + 1);
+	while (*p) {
+		if ((len >= 4) && (*p == '\\') && (*(p+1) == 'x')) {
+			*n++ = (char) hex2byte (p + 2);
+			p += 4;
+			len -= 4;
+		} else {
+			*n++ = *p++;
+			len--;
 		}
-
-		if (err || !parent) {
-			g_warning ("Error getting originating device info from HAL: %s",
-			           err ? err->message : "unknown error");
-			if (err)
-				g_error_free (err);
-		}
-	} else {
-		if (!dbus_g_proxy_call (proxy, "GetPropertyString", &err,
-								G_TYPE_STRING, "info.parent",
-								G_TYPE_INVALID,
-								G_TYPE_STRING, &parent,
-								G_TYPE_INVALID)) {
-			g_warning ("Error getting parent device info from HAL: %s", err->message);
-			g_error_free (err);
-	    }
 	}
 
-	g_object_unref (proxy);
-	return parent;
-}
-
-static char *
-proxy_get_string (DBusGProxy *proxy,
-                  const char *property,
-                  gboolean warn)
-{
-	GError *error = NULL;
-	char *result = NULL;
-
-	g_return_val_if_fail (proxy != NULL, NULL);
-	g_return_val_if_fail (property != NULL, NULL);
-
-	if (dbus_g_proxy_call (proxy, "GetPropertyString", &error,
-	                       G_TYPE_STRING, property, G_TYPE_INVALID,
-	                       G_TYPE_STRING, &result, G_TYPE_INVALID))
-		return result;
-
-	if (warn) {
-		g_warning ("Error getting HAL property '%s' from device '%s': %s",
-		           property, dbus_g_proxy_get_path (proxy),
-		           error ? error->message : "unknown");
-	}
-	g_error_free (error);
-	return NULL;
-}
-
-static gboolean
-get_product_and_vendor (DBusGConnection *connection,
-                        const char *udi,
-                        char **product,
-                        char **vendor)
-{
-	DBusGProxy *proxy;
-	char *tmp_product = NULL;
-	char *tmp_vendor = NULL;
-	char *subsys = NULL;
-	gboolean product_fallback = TRUE, vendor_fallback = TRUE;
-	gboolean warn = FALSE;
-
-	g_return_val_if_fail (connection != NULL, FALSE);
-	g_return_val_if_fail (udi != NULL, FALSE);
-
-	g_return_val_if_fail (product != NULL, FALSE);
-	g_return_val_if_fail (*product == NULL, FALSE);
-
-	g_return_val_if_fail (vendor != NULL, FALSE);
-	g_return_val_if_fail (*vendor == NULL, FALSE);
-
-	proxy = dbus_g_proxy_new_for_name (connection, "org.freedesktop.Hal", udi, "org.freedesktop.Hal.Device");
-	if (!proxy)
-		return FALSE;
-
-	subsys = proxy_get_string (proxy, "info.subsystem", warn);
-	if (subsys && !strcmp (subsys, "pci")) {
-		tmp_product = proxy_get_string (proxy, "pci.subsys_product", warn);
-		if (tmp_product)
-			product_fallback = FALSE;
-
-		tmp_vendor = proxy_get_string (proxy, "pci.subsys_vendor", warn);
-		if (tmp_vendor)
-			vendor_fallback = FALSE;
-	}
-	g_free (subsys);
-
-	if (product_fallback)
-		tmp_product = proxy_get_string (proxy, "info.product", warn);
-	if (vendor_fallback)
-		tmp_vendor = proxy_get_string (proxy, "info.vendor", warn);
-
-	if (tmp_product && tmp_vendor) {
-		*product = tmp_product;
-		*vendor = tmp_vendor;
-	} else {
-		g_free (tmp_product);
-		g_free (tmp_vendor);
-	}
-	g_object_unref (proxy);
-
-	return (*product && *vendor) ? TRUE : FALSE;
+	return unescaped;
 }
 
 static void
 nm_device_update_description (NMDevice *device)
 {
 	NMDevicePrivate *priv;
-	DBusGConnection *connection;
-	const char *udi;
-	char *orig_dev_udi = NULL;
-	char *parent_udi = NULL;
+	const char *subsys[3] = { "net", "tty", NULL };
+	GUdevDevice *udev_device = NULL, *tmpdev;
+	const char *ifname;
+	guint32 count = 0;
+	const char *vendor, *model;
 
 	g_return_if_fail (NM_IS_DEVICE (device));
 	priv = NM_DEVICE_GET_PRIVATE (device);
 
+	if (!priv->client) {
+		priv->client = g_udev_client_new (subsys);
+		if (!priv->client)
+			return;
+	}
+
+	ifname = nm_device_get_iface (device);
+	if (!ifname)
+		return;
+
+	if (NM_IS_DEVICE_ETHERNET (device) || NM_IS_DEVICE_WIFI (device))
+		udev_device = g_udev_client_query_by_subsystem_and_name (priv->client, "net", ifname);
+	else if (NM_IS_GSM_DEVICE (device) || NM_IS_CDMA_DEVICE (device))
+		udev_device = g_udev_client_query_by_subsystem_and_name (priv->client, "tty", ifname);
+	if (!udev_device)
+		return;
+
 	g_free (priv->product);
 	priv->product = NULL;
 	g_free (priv->vendor);
 	priv->vendor = NULL;
 
-	connection = nm_object_get_connection (NM_OBJECT (device));
-	g_return_if_fail (connection != NULL);
+	/* Walk up the chain of the device and its parents a few steps to grab
+	 * vendor and device ID information off it.
+	 */
+	tmpdev = udev_device;
+	while ((count++ < 3) && tmpdev && (!priv->vendor || !priv->product)) {
+		if (!priv->vendor)
+			priv->vendor = get_decoded_property (tmpdev, "ID_VENDOR_ENC");
 
-	/* First, get the udi of the originating device */
-	udi = nm_device_get_udi (device);
-	orig_dev_udi = get_ancestor_device (device, connection, udi, TRUE);
+		if (!priv->product)
+			priv->product = get_decoded_property (tmpdev, "ID_MODEL_ENC");
 
-	/* Get product and vendor off the originating device if possible */
-	if (!get_product_and_vendor (connection, orig_dev_udi, &priv->product, &priv->vendor)) {
-		 /* Try the parent of the originating device */
-		parent_udi = get_ancestor_device (device, connection, orig_dev_udi, FALSE);
-		if (parent_udi)
-			get_product_and_vendor (connection, parent_udi, &priv->product, &priv->vendor);
-		g_free (parent_udi);
+		tmpdev = g_udev_device_get_parent (tmpdev);
 	}
 
-	g_free (orig_dev_udi);
+	/* If we didn't get strings directly from the device, try database strings */
+	tmpdev = udev_device;
+	count = 0;
+	while ((count++ < 3) && tmpdev && (!priv->vendor || !priv->product)) {
+		if (!priv->vendor) {
+			vendor = g_udev_device_get_property (tmpdev, "ID_VENDOR_FROM_DATABASE");
+			if (vendor)
+				priv->vendor = g_strdup (vendor);
+		}
+
+		if (!priv->product) {
+			model = g_udev_device_get_property (tmpdev, "ID_MODEL_FROM_DATABASE");
+			if (model)
+				priv->product = g_strdup (model);
+		}
+
+		tmpdev = g_udev_device_get_parent (tmpdev);
+	}
 
 	_nm_object_queue_notify (NM_OBJECT (device), NM_DEVICE_VENDOR);
 	_nm_object_queue_notify (NM_OBJECT (device), NM_DEVICE_PRODUCT);
@@ -1010,3 +1162,59 @@ nm_device_get_vendor (NMDevice *device)
 	return priv->vendor;
 }
 
+typedef struct {
+	NMDevice *device;
+	NMDeviceDeactivateFn fn;
+	gpointer user_data;
+} DeactivateInfo;
+
+static void
+deactivate_cb (DBusGProxy *proxy,
+               GError *error,
+               gpointer user_data)
+{
+	DeactivateInfo *info = user_data;
+
+	if (info->fn)
+		info->fn (info->device, error, info->user_data);
+	else if (error) {
+		g_warning ("%s: device %s deactivation failed: (%d) %s",
+		           __func__,
+		           nm_object_get_path (NM_OBJECT (info->device)),
+		           error ? error->code : -1,
+		           error && error->message ? error->message : "(unknown)");
+	}
+
+	g_object_unref (info->device);
+	g_slice_free (DeactivateInfo, info);
+}
+
+/**
+ * nm_device_disconnect:
+ * @device: a #NMDevice
+ * @callback: callback to be called when disconnect operation completes
+ * @user_data: caller-specific data passed to @callback
+ *
+ * Disconnects the device if currently connected, and prevents the device from
+ * automatically connecting to networks until the next manual network connection
+ * request.
+ **/
+void
+nm_device_disconnect (NMDevice *device,
+                      NMDeviceDeactivateFn callback,
+                      gpointer user_data)
+{
+	DeactivateInfo *info;
+
+	g_return_if_fail (NM_IS_DEVICE (device));
+
+	info = g_slice_new (DeactivateInfo);
+	info->fn = callback;
+	info->user_data = user_data;
+	info->device = g_object_ref (device);
+
+	org_freedesktop_NetworkManager_Device_disconnect_async (NM_DEVICE_GET_PRIVATE (device)->proxy,
+	                                                        deactivate_cb,
+	                                                        info);
+}
+
