@@ -16,7 +16,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  * Copyright (C) 2004 - 2005 Colin Walters <walters@redhat.com>
- * Copyright (C) 2004 - 2008 Red Hat, Inc.
+ * Copyright (C) 2004 - 2010 Red Hat, Inc.
  * Copyright (C) 2005 - 2008 Novell, Inc.
  *   and others
  */
@@ -38,8 +38,9 @@
 
 #include "nm-named-manager.h"
 #include "nm-ip4-config.h"
-#include "nm-utils.h"
-#include "NetworkManagerSystem.h"
+#include "nm-ip6-config.h"
+#include "nm-logging.h"
+#include "nm-system.h"
 #include "NetworkManagerUtils.h"
 
 #ifdef HAVE_SELINUX
@@ -60,11 +61,11 @@ G_DEFINE_TYPE(NMNamedManager, nm_named_manager, G_TYPE_OBJECT)
 
 
 struct NMNamedManagerPrivate {
-	NMIP4Config *   vpn_config;
-	NMIP4Config *   device_config;
-	GSList *        configs;
-
-	gboolean disposed;
+	NMIP4Config *ip4_vpn_config;
+	NMIP4Config *ip4_device_config;
+	NMIP6Config *ip6_vpn_config;
+	NMIP6Config *ip6_device_config;
+	GSList *configs;
 };
 
 
@@ -73,11 +74,10 @@ nm_named_manager_get (void)
 {
 	static NMNamedManager * singleton = NULL;
 
-	if (!singleton) {
+	if (!singleton)
 		singleton = NM_NAMED_MANAGER (g_object_new (NM_TYPE_NAMED_MANAGER, NULL));
-	} else {
+	else
 		g_object_ref (singleton);
-	}
 
 	g_assert (singleton);
 	return singleton;
@@ -94,29 +94,97 @@ nm_named_manager_error_quark (void)
 	return quark;
 }
 
+typedef struct {
+	GPtrArray *nameservers;
+	const char *domain;
+	GPtrArray *searches;
+} NMResolvConfData;
+
 static void
-merge_one_ip4_config (NMIP4Config *dst, NMIP4Config *src)
+add_string_item (GPtrArray *array, const char *str)
 {
-	guint32 num, num_domains, i;
+	int i;
+
+	g_return_if_fail (array != NULL);
+	g_return_if_fail (str != NULL);
+
+	/* Check for dupes before adding */
+	for (i = 0; i < array->len; i++) {
+		const char *candidate = g_ptr_array_index (array, i);
+
+		if (candidate && !strcmp (candidate, str))
+			return;
+	}
+
+	/* No dupes, add the new item */
+	g_ptr_array_add (array, g_strdup (str));
+}
+
+static void
+merge_one_ip4_config (NMResolvConfData *rc, NMIP4Config *src)
+{
+	guint32 num, i;
 
 	num = nm_ip4_config_get_num_nameservers (src);
-	for (i = 0; i < num; i++)
-		nm_ip4_config_add_nameserver (dst, nm_ip4_config_get_nameserver (src, i));
+	for (i = 0; i < num; i++) {
+		struct in_addr addr;
+		char buf[INET_ADDRSTRLEN];
 
-	num_domains = nm_ip4_config_get_num_domains (src);
-	for (i = 0; i < num_domains; i++)
-		nm_ip4_config_add_domain (dst, nm_ip4_config_get_domain (src, i));
+		addr.s_addr = nm_ip4_config_get_nameserver (src, i);
+		if (inet_ntop (AF_INET, &addr, buf, INET_ADDRSTRLEN) > 0)
+			add_string_item (rc->nameservers, buf);
+	}
+
+	num = nm_ip4_config_get_num_domains (src);
+	for (i = 0; i < num; i++) {
+		const char *domain;
+
+		domain = nm_ip4_config_get_domain (src, i);
+		if (!rc->domain)
+			rc->domain = domain;
+		add_string_item (rc->searches, domain);
+	}
 
 	num = nm_ip4_config_get_num_searches (src);
 	for (i = 0; i < num; i++)
-		nm_ip4_config_add_search (dst, nm_ip4_config_get_search (src, i));
+		add_string_item (rc->searches, nm_ip4_config_get_search (src, i));
+}
 
-	/* Add the 'domain' list to searches as well since overloading the
-	 * 'domain_name' DHCP field used to be the way you got searches
-	 * into resolv.conf.
-	 */
-	for (i = 0; i < num_domains; i++)
-		nm_ip4_config_add_search (dst, nm_ip4_config_get_domain (src, i));
+static void
+merge_one_ip6_config (NMResolvConfData *rc, NMIP6Config *src)
+{
+	guint32 num, i;
+
+	num = nm_ip6_config_get_num_nameservers (src);
+	for (i = 0; i < num; i++) {
+		const struct in6_addr *addr;
+		char buf[INET6_ADDRSTRLEN];
+
+		addr = nm_ip6_config_get_nameserver (src, i);
+
+		/* inet_ntop is probably supposed to do this for us, but it doesn't */
+		if (IN6_IS_ADDR_V4MAPPED (addr)) {
+			if (inet_ntop (AF_INET, &(addr->s6_addr32[3]), buf, INET_ADDRSTRLEN) > 0)
+				add_string_item (rc->nameservers, buf);
+		} else {
+			if (inet_ntop (AF_INET6, addr, buf, INET6_ADDRSTRLEN) > 0)
+				add_string_item (rc->nameservers, buf);
+		}
+	}
+
+	num = nm_ip6_config_get_num_domains (src);
+	for (i = 0; i < num; i++) {
+		const char *domain;
+
+		domain = nm_ip6_config_get_domain (src, i);
+		if (!rc->domain)
+			rc->domain = domain;
+		add_string_item (rc->searches, domain);
+	}
+
+	num = nm_ip6_config_get_num_searches (src);
+	for (i = 0; i < num; i++)
+		add_string_item (rc->searches, nm_ip6_config_get_search (src, i));
 }
 
 
@@ -145,7 +213,7 @@ run_netconfig (GError **error, gint *stdin_fd)
 	argv[4] = NULL;
 
 	tmp = g_strjoinv (" ", argv);
-	nm_debug ("Spawning '%s'", tmp);
+	nm_log_debug (LOGD_DNS, "spawning '%s'", tmp);
 	g_free (tmp);
 
 	if (!g_spawn_async_with_pipes (NULL, argv, NULL, 0, netconfig_child_setup,
@@ -162,7 +230,7 @@ write_to_netconfig (gint fd, const char *key, const char *value)
 	int x;
 
 	str = g_strdup_printf ("%s='%s'\n", key, value);
-	nm_debug ("Writing to netconfig: %s", str);
+	nm_log_debug (LOGD_DNS, "writing to netconfig: %s", str);
 	x = write (fd, str, strlen (str));
 	g_free (str);
 }
@@ -183,6 +251,15 @@ dispatch_netconfig (const char *domain,
 	if (pid < 0)
 		return FALSE;
 
+	// FIXME: this is wrong. We are not writing out the iface-specific
+	// resolv.conf data, we are writing out an already-fully-merged
+	// resolv.conf. Assuming netconfig works in the obvious way, then
+	// there are various failure modes, such as, eg, bringing up a VPN on
+	// eth0, then bringing up wlan0, then bringing down the VPN. Because
+	// NMNamedManager would have claimed that the VPN DNS server was also
+	// part of the wlan0 config, it will remain in resolv.conf after the
+	// VPN goes down, even though it is presumably no longer reachable
+	// at that point.
 	write_to_netconfig (fd, "INTERFACE", iface);
 
 	if (searches) {
@@ -278,7 +355,7 @@ write_resolv_conf (FILE *f, const char *domain,
 	}
 
 	if (fprintf (f, "%s%s%s",
-		     domain_str ? domain_str : "",
+	             domain_str ? domain_str : "",
 	             searches_str ? searches_str : "",
 	             nameservers_str ? nameservers_str : "") != -1)
 		retval = TRUE;
@@ -307,7 +384,7 @@ dispatch_resolvconf (const char *domain,
 
 	if (domain || searches || nameservers) {
 		cmd = g_strconcat (RESOLVCONF_PATH, " -a ", "NetworkManager", NULL);
-		nm_info ("(%s): writing resolv.conf to %s", iface, RESOLVCONF_PATH);
+		nm_log_info (LOGD_DNS, "(%s): writing resolv.conf to %s", iface, RESOLVCONF_PATH);
 		if ((f = popen (cmd, "w")) == NULL)
 			g_set_error (error,
 				     NM_NAMED_MANAGER_ERROR,
@@ -321,7 +398,7 @@ dispatch_resolvconf (const char *domain,
 		}
 	} else {
 		cmd = g_strconcat (RESOLVCONF_PATH, " -d ", "NetworkManager", NULL);
-		nm_info ("(%s): removing resolv.conf from %s", iface, RESOLVCONF_PATH);
+		nm_log_info (LOGD_DNS, "(%s): removing resolv.conf from %s", iface, RESOLVCONF_PATH);
 		if (nm_spawn_process (cmd) == 0)
 			retval = TRUE;
 	}
@@ -339,14 +416,29 @@ update_resolv_conf (const char *domain,
                     const char *iface,
                     GError **error)
 {
-	const char *tmp_resolv_conf = RESOLV_CONF ".tmp";
-	char tmp_resolv_conf_realpath [PATH_MAX];
+	char *tmp_resolv_conf;
+	char *tmp_resolv_conf_realpath;
+	char *resolv_conf_realpath;
 	FILE *f;
 	int do_rename = 1;
 	int old_errno = 0;
 
-	if (!realpath (tmp_resolv_conf, tmp_resolv_conf_realpath))
-		strcpy (tmp_resolv_conf_realpath, tmp_resolv_conf);
+	g_return_val_if_fail (error != NULL, FALSE);
+
+	/* Find the real path of resolv.conf; it could be a symlink to something */
+	resolv_conf_realpath = realpath (RESOLV_CONF, NULL);
+	if (!resolv_conf_realpath)
+		resolv_conf_realpath = strdup (RESOLV_CONF);
+
+	/* Build up the real path for the temp resolv.conf that we're about to
+	 * write out.
+	 */
+	tmp_resolv_conf = g_strdup_printf ("%s.tmp", resolv_conf_realpath);
+	tmp_resolv_conf_realpath = realpath (tmp_resolv_conf, NULL);
+	if (!tmp_resolv_conf_realpath)
+		tmp_resolv_conf_realpath = strdup (tmp_resolv_conf);
+	g_free (tmp_resolv_conf);
+	tmp_resolv_conf = NULL;
 
 	if ((f = fopen (tmp_resolv_conf_realpath, "w")) == NULL) {
 		do_rename = 0;
@@ -360,8 +452,11 @@ update_resolv_conf (const char *domain,
 			             g_strerror (old_errno),
 			             RESOLV_CONF,
 			             g_strerror (errno));
-			return FALSE;
+			goto out;
 		}
+		/* Update tmp_resolv_conf_realpath so the error message on fclose()
+		 * failure will be correct.
+		 */
 		strcpy (tmp_resolv_conf_realpath, RESOLV_CONF);
 	}
 
@@ -369,72 +464,47 @@ update_resolv_conf (const char *domain,
 
 	if (fclose (f) < 0) {
 		if (*error == NULL) {
+			/* only set an error here if write_resolv_conf() was successful,
+			 * since its error is more important.
+			 */
 			g_set_error (error,
-					   NM_NAMED_MANAGER_ERROR,
-					   NM_NAMED_MANAGER_ERROR_SYSTEM,
-					   "Could not close %s: %s\n",
-					   tmp_resolv_conf_realpath,
-					   g_strerror (errno));
+			             NM_NAMED_MANAGER_ERROR,
+			             NM_NAMED_MANAGER_ERROR_SYSTEM,
+			             "Could not close %s: %s\n",
+			             tmp_resolv_conf_realpath,
+			             g_strerror (errno));
 		}
 	}
 
+	/* Don't rename the tempfile over top of the existing resolv.conf if there
+	 * was an error writing it out.
+	 */
 	if (*error == NULL && do_rename) {
-		if (rename (tmp_resolv_conf, RESOLV_CONF) < 0) {
+		if (rename (tmp_resolv_conf_realpath, resolv_conf_realpath) < 0) {
 			g_set_error (error,
-					   NM_NAMED_MANAGER_ERROR,
-					   NM_NAMED_MANAGER_ERROR_SYSTEM,
-					   "Could not replace " RESOLV_CONF ": %s\n",
-					   g_strerror (errno));
+			             NM_NAMED_MANAGER_ERROR,
+			             NM_NAMED_MANAGER_ERROR_SYSTEM,
+			             "Could not replace " RESOLV_CONF ": %s\n",
+			             g_strerror (errno));
 		}
 	}
 
+out:
+	free (tmp_resolv_conf_realpath);
+	free (resolv_conf_realpath);
 	return *error ? FALSE : TRUE;
 }
 
-static char **
-compute_searches (guint32 num, NMIP4Config *config, gboolean searches)
-{
-	GPtrArray *array;
-	size_t len, elem_len;
-	const char *elem;
-	int i;
-
-	/* Search list is limited to 6 domains total per 'man resolv.conf' */
-	if (num > 6)
-		num = 6;
-
-	array = g_ptr_array_sized_new (num + 1);
-	for (i = 0, len = 0; i < num; i++) {
-		elem = searches ? nm_ip4_config_get_search (config, i)
-			        : nm_ip4_config_get_domain (config, i);
-		elem_len = strlen (elem);
-
-		/* The search list is limited to 256 characters per 'man resolv.conf' */
-		if (len + elem_len > 255)
-			break;
-
-		g_ptr_array_add (array, g_strdup (elem));
-		len += elem_len + 1;  /* +1 for spaces */
-	}
-
-	g_ptr_array_add (array, NULL);
-	return (char **) g_ptr_array_free (array, FALSE);
-}
-
 static gboolean
 rewrite_resolv_conf (NMNamedManager *mgr, const char *iface, GError **error)
 {
 	NMNamedManagerPrivate *priv;
-	NMIP4Config *composite;
+	NMResolvConfData rc;
 	GSList *iter;
-	GPtrArray *array;
 	const char *domain = NULL;
 	char **searches = NULL;
 	char **nameservers = NULL;
-	int num_domains;
-	int num_searches;
-	int num_nameservers;
-	int i;
+	int num, i, len;
 	gboolean success = FALSE;
 
 	g_return_val_if_fail (error != NULL, FALSE);
@@ -442,66 +512,62 @@ rewrite_resolv_conf (NMNamedManager *mgr, const char *iface, GError **error)
 
 	priv = NM_NAMED_MANAGER_GET_PRIVATE (mgr);
 
-	/* Construct the composite config from all the currently active IP4Configs */
-	composite = nm_ip4_config_new ();
+	rc.nameservers = g_ptr_array_new ();
+	rc.domain = NULL;
+	rc.searches = g_ptr_array_new ();
 
-	if (priv->vpn_config)
-		merge_one_ip4_config (composite, priv->vpn_config);
+	if (priv->ip4_vpn_config)
+		merge_one_ip4_config (&rc, priv->ip4_vpn_config);
+	if (priv->ip4_device_config)
+		merge_one_ip4_config (&rc, priv->ip4_device_config);
 
-	if (priv->device_config)
-		merge_one_ip4_config (composite, priv->device_config);
+	if (priv->ip6_vpn_config)
+		merge_one_ip6_config (&rc, priv->ip6_vpn_config);
+	if (priv->ip6_device_config)
+		merge_one_ip6_config (&rc, priv->ip6_device_config);
 
 	for (iter = priv->configs; iter; iter = g_slist_next (iter)) {
-		NMIP4Config *config = NM_IP4_CONFIG (iter->data);
-
-		if ((config == priv->vpn_config) || (config == priv->device_config))
+		if (   (iter->data == priv->ip4_vpn_config)
+		    || (iter->data == priv->ip4_device_config)
+		    || (iter->data == priv->ip6_vpn_config)
+		    || (iter->data == priv->ip6_device_config))
 			continue;
 
-		merge_one_ip4_config (composite, config);
+		if (NM_IS_IP4_CONFIG (iter->data)) {
+			NMIP4Config *config = NM_IP4_CONFIG (iter->data);
+
+			merge_one_ip4_config (&rc, config);
+		} else if (NM_IS_IP6_CONFIG (iter->data)) {
+			NMIP6Config *config = NM_IP6_CONFIG (iter->data);
+
+			merge_one_ip6_config (&rc, config);
+		} else
+			g_assert_not_reached ();
 	}
 
-	/* Some DHCP servers like to put multiple search paths into the domain
-	 * option as the domain-search option wasn't around in the first RFC.
-	 * We should try and support these old servers as best we can. */
+	domain = rc.domain;
 
-	num_domains = nm_ip4_config_get_num_domains (composite);
-	num_searches = nm_ip4_config_get_num_searches (composite);
-
-	/* Domain */
-	if (num_domains > 0)
-		domain = nm_ip4_config_get_domain (composite, 0);
-
-	/* Searches */
-	if (num_searches > 0)
-		searches = compute_searches (num_searches, composite, TRUE);
-	else if (num_domains > 0)
-		searches = compute_searches (num_searches, composite, FALSE);
-
-	/* Name servers */
-	num_nameservers = nm_ip4_config_get_num_nameservers (composite);
-	if (num_nameservers > 0) {
-		array = g_ptr_array_sized_new (num_nameservers + 1);
-		for (i = 0; i < num_nameservers; i++) {
-			struct in_addr addr;
-			char *buf;
-
-			addr.s_addr = nm_ip4_config_get_nameserver (composite, i);
-			buf = g_malloc0 (ADDR_BUF_LEN);
-			if (!buf)
-				continue;
-
-			if (inet_ntop (AF_INET, &addr, buf, ADDR_BUF_LEN))
-				g_ptr_array_add (array, buf);
-			else
-				nm_warning ("%s: error converting IP4 address 0x%X",
-						  __func__, ntohl (addr.s_addr));
-		}
-
-		g_ptr_array_add (array, NULL);
-		nameservers = (char **) g_ptr_array_free (array, FALSE);
+	/* Per 'man resolv.conf', the search list is limited to 6 domains
+	 * totalling 256 characters.
+	 */
+	num = MIN (rc.searches->len, 6);
+	for (i = 0, len = 0; i < num; i++) {
+		len += strlen (rc.searches->pdata[i]) + 1; /* +1 for spaces */
+		if (len > 256)
+			break;
 	}
+	g_ptr_array_set_size (rc.searches, i);
+	if (rc.searches->len) {
+		g_ptr_array_add (rc.searches, NULL);
+		searches = (char **) g_ptr_array_free (rc.searches, FALSE);
+	} else
+		g_ptr_array_free (rc.searches, TRUE);
 
-	g_object_unref (composite);
+	if (rc.nameservers->len) {
+		g_ptr_array_add (rc.nameservers, NULL);
+		nameservers = (char **) g_ptr_array_free (rc.nameservers, FALSE);
+	} else
+		g_ptr_array_free (rc.nameservers, TRUE);
 
 #ifdef RESOLVCONF_PATH
 	success = dispatch_resolvconf (domain, searches, nameservers, iface, error);
@@ -518,17 +584,19 @@ rewrite_resolv_conf (NMNamedManager *mgr, const char *iface, GError **error)
 	if (success)
 		nm_system_update_dns ();
 
-	g_strfreev (searches);
-	g_strfreev (nameservers);
+	if (searches)
+		g_strfreev (searches);
+	if (nameservers)
+		g_strfreev (nameservers);
 
 	return success;
 }
 
 gboolean
 nm_named_manager_add_ip4_config (NMNamedManager *mgr,
-						   const char *iface,
-                                 NMIP4Config *config,
-                                 NMNamedIPConfigType cfg_type)
+								 const char *iface,
+								 NMIP4Config *config,
+								 NMNamedIPConfigType cfg_type)
 {
 	NMNamedManagerPrivate *priv;
 	GError *error = NULL;
@@ -541,10 +609,10 @@ nm_named_manager_add_ip4_config (NMNamedManager *mgr,
 
 	switch (cfg_type) {
 	case NM_NAMED_IP_CONFIG_TYPE_VPN:
-		priv->vpn_config = config;
+		priv->ip4_vpn_config = config;
 		break;
 	case NM_NAMED_IP_CONFIG_TYPE_BEST_DEVICE:
-		priv->device_config = config;
+		priv->ip4_device_config = config;
 		break;
 	default:
 		break;
@@ -555,7 +623,7 @@ nm_named_manager_add_ip4_config (NMNamedManager *mgr,
 		priv->configs = g_slist_append (priv->configs, g_object_ref (config));
 
 	if (!rewrite_resolv_conf (mgr, iface, &error)) {
-		nm_warning ("Could not commit DNS changes.  Error: '%s'", error ? error->message : "(none)");
+		nm_log_warn (LOGD_DNS, "could not commit DNS changes: '%s'", error ? error->message : "(none)");
 		g_error_free (error);
 	}
 
@@ -564,8 +632,8 @@ nm_named_manager_add_ip4_config (NMNamedManager *mgr,
 
 gboolean
 nm_named_manager_remove_ip4_config (NMNamedManager *mgr,
-							 const char *iface,
-							 NMIP4Config *config)
+									const char *iface,
+									NMIP4Config *config)
 {
 	NMNamedManagerPrivate *priv;
 	GError *error = NULL;
@@ -582,16 +650,91 @@ nm_named_manager_remove_ip4_config (NMNamedManager *mgr,
 
 	priv->configs = g_slist_remove (priv->configs, config);
 
-	if (config == priv->vpn_config)
-		priv->vpn_config = NULL;
+	if (config == priv->ip4_vpn_config)
+		priv->ip4_vpn_config = NULL;
+	if (config == priv->ip4_device_config)
+		priv->ip4_device_config = NULL;
 
-	if (config == priv->device_config)
-		priv->device_config = NULL;
+	g_object_unref (config);
+
+	if (!rewrite_resolv_conf (mgr, iface, &error)) {
+		nm_log_warn (LOGD_DNS, "could not commit DNS changes: '%s'", error ? error->message : "(none)");
+		if (error)
+			g_error_free (error);
+	}
+
+	return TRUE;
+}
+
+gboolean
+nm_named_manager_add_ip6_config (NMNamedManager *mgr,
+								 const char *iface,
+								 NMIP6Config *config,
+								 NMNamedIPConfigType cfg_type)
+{
+	NMNamedManagerPrivate *priv;
+	GError *error = NULL;
+
+	g_return_val_if_fail (mgr != NULL, FALSE);
+	g_return_val_if_fail (iface != NULL, FALSE);
+	g_return_val_if_fail (config != NULL, FALSE);
+
+	priv = NM_NAMED_MANAGER_GET_PRIVATE (mgr);
+
+	switch (cfg_type) {
+	case NM_NAMED_IP_CONFIG_TYPE_VPN:
+		/* FIXME: not quite yet... */
+		g_return_val_if_fail (cfg_type != NM_NAMED_IP_CONFIG_TYPE_VPN, FALSE);
+		priv->ip6_vpn_config = config;
+		break;
+	case NM_NAMED_IP_CONFIG_TYPE_BEST_DEVICE:
+		priv->ip6_device_config = config;
+		break;
+	default:
+		break;
+	}
+
+	/* Don't allow the same zone added twice */
+	if (!g_slist_find (priv->configs, config))
+		priv->configs = g_slist_append (priv->configs, g_object_ref (config));
+
+	if (!rewrite_resolv_conf (mgr, iface, &error)) {
+		nm_log_warn (LOGD_DNS, "could not commit DNS changes: '%s'", error ? error->message : "(none)");
+		g_error_free (error);
+	}
+
+	return TRUE;
+}
+
+gboolean
+nm_named_manager_remove_ip6_config (NMNamedManager *mgr,
+									const char *iface,
+									NMIP6Config *config)
+{
+	NMNamedManagerPrivate *priv;
+	GError *error = NULL;
+
+	g_return_val_if_fail (mgr != NULL, FALSE);
+	g_return_val_if_fail (iface != NULL, FALSE);
+	g_return_val_if_fail (config != NULL, FALSE);
+
+	priv = NM_NAMED_MANAGER_GET_PRIVATE (mgr);
+
+	/* Can't remove it if it wasn't in the list to begin with */
+	if (!g_slist_find (priv->configs, config))
+		return FALSE;
+
+	priv->configs = g_slist_remove (priv->configs, config);
+
+	if (config == priv->ip6_vpn_config)
+		priv->ip6_vpn_config = NULL;
+	if (config == priv->ip6_device_config)
+		priv->ip6_device_config = NULL;
 
 	g_object_unref (config);	
 
 	if (!rewrite_resolv_conf (mgr, iface, &error)) {
-		nm_warning ("Could not commit DNS changes.  Error: '%s'", error ? error->message : "(none)");
+		nm_log_warn (LOGD_DNS, "could not commit DNS changes: '%s'", error ? error->message : "(none)");
 		if (error)
 			g_error_free (error);
 	}
